@@ -22,7 +22,6 @@ from torch.utils.data import TensorDataset, random_split
 from tqdm import tqdm
 
 from neural_semigroups.constants import CAYLEY_DATABASE_PATH, CURRENT_DEVICE
-from neural_semigroups.denoising_autoencoder import MagmaDAE
 from neural_semigroups.magma import Magma
 from neural_semigroups.utils import (
     get_equivalent_magmas,
@@ -72,16 +71,15 @@ class CayleyDatabase:
         :param cayley_table: a partially filled Cayley table (unknow entries are filled by ``-1``)
         :returns: whether the input is correct
         """
-        correct = True
         if isinstance(cayley_table, list):
             # pylint: disable=not-callable
             table = torch.tensor(cayley_table)
-            if table.shape != torch.Size([self.cardinality, self.cardinality]):
-                correct = False
-            elif table.dtype != torch.long:
-                correct = False
-            elif table.max() >= self.cardinality or table.min() < -1:
-                correct = False
+            correct = (
+                table.shape == torch.Size([self.cardinality, self.cardinality])
+                and table.dtype == torch.long
+                and table.max() < self.cardinality
+                and table.min() >= -1
+            )
         else:
             correct = False
         return correct
@@ -122,8 +120,6 @@ class CayleyDatabase:
         :param cayley_table: a partially filled Cayley table (unknow entries are filled by ``-1``)
         :returns: a tuple: (most probable completion, probabilistic cube)
         """
-        if isinstance(self.model, MagmaDAE):
-            self.model.apply_corruption = False
         self.model.eval()
         if not self._check_input(cayley_table):
             raise ValueError(
