@@ -32,6 +32,7 @@ from ignite.engine import (
     create_supervised_trainer,
 )
 from ignite.handlers import EarlyStopping, ModelCheckpoint
+from ignite.metrics import RunningAverage
 from ignite.metrics.loss import Loss
 from torch.nn import Module
 from torch.optim import Adam
@@ -43,11 +44,32 @@ from neural_semigroups.cayley_database import CayleyDatabase
 from neural_semigroups.constants import CURRENT_DEVICE
 from neural_semigroups.magma import Magma
 from neural_semigroups.precise_guess_loss import PreciseGuessLoss
-from neural_semigroups.utils import get_newest_file
+from neural_semigroups.utils import corrupt_input, get_newest_file
+
+
+def generate_features_and_labels(
+    cayley_cubes: torch.Tensor, dropout_rate: float
+) -> Tuple[torch.Tensor, torch.Tensor]:
+    """
+    a function-helper to generate labels as features to which we apply dropout
+
+    :param cayley_cubes: tensors to create features and labels for training
+    :param dropout_rate: dropout rate to apply for generating labels
+    :returns: a pair of (features, labels) for training
+    """
+    features_list = list()
+    for cayley_table in tqdm(cayley_cubes):
+        cube = Magma(cayley_table).probabilistic_cube
+        features_list.append(cube)
+    features = torch.stack(features_list)
+    return corrupt_input(features, dropout_rate), features
 
 
 def load_database_as_cubes(
-    cardinality: int, train_size: int, validation_size: int
+    cardinality: int,
+    train_size: int,
+    validation_size: int,
+    dropout_rate: float,
 ) -> Tuple[
     torch.Tensor,
     torch.Tensor,
@@ -62,11 +84,11 @@ def load_database_as_cubes(
     :param cardinality: cardinality of Cayley database (from ``smallsemi``)
     :param train_size: number of tables for training
     :param validation_size: number of tables for validation
+    :param dropout_rate: droupout is applied only to validation and test sets
     :returns: three arrays of probability Cayley cubes (train, validation, test
     ) and three arrays of labels for them
     """
-    cayley_db = CayleyDatabase(cardinality)
-    train, validation, test = cayley_db.train_test_split(
+    train, validation, test = CayleyDatabase(cardinality).train_test_split(
         train_size, validation_size
     )
     train.augment_by_equivalent_tables()
@@ -75,21 +97,20 @@ def load_database_as_cubes(
     train_cubes = list()
     for cayley_table in tqdm(train.database, desc="generating train cubes"):
         train_cubes.append(Magma(cayley_table).probabilistic_cube)
-    validation_cubes = list()
-    for cayley_table in tqdm(
-        validation.database, desc="generating validation cubes"
-    ):
-        validation_cubes.append(Magma(cayley_table).probabilistic_cube)
-    test_cubes = list()
-    for cayley_table in tqdm(test.database, desc="generating test cubes"):
-        test_cubes.append(Magma(cayley_table).probabilistic_cube)
+    validation_cubes, validation_labels = generate_features_and_labels(
+        validation.database, dropout_rate
+    )
+    test_cubes, test_labels = generate_features_and_labels(
+        test.database, dropout_rate
+    )
+    train_labels = torch.stack(train_cubes)
     return (
-        torch.stack(train_cubes),
-        torch.stack(validation_cubes),
-        torch.stack(test_cubes),
-        train.labels,
-        validation.labels,
-        test.labels,
+        train_labels,
+        validation_cubes,
+        test_cubes,
+        train_labels,
+        validation_labels,
+        test_labels,
     )
 
 
@@ -255,10 +276,10 @@ def get_tensorboard_logger(
     )
     training_loss = OutputHandler(
         "training",
-        ["loss"],
+        ["running_loss"],
         global_step_transform=global_step_from_engine(trainer),
     )
-    tb_logger.attach(evaluators.train, training_loss, Events.COMPLETED)
+    tb_logger.attach(trainer, training_loss, Events.EPOCH_COMPLETED)
     validation_loss = OutputHandler(
         "validation",
         ["loss", "associative_ratio", "guessed_ratio"],
@@ -274,6 +295,33 @@ def get_tensorboard_logger(
     return tb_logger
 
 
+def get_trainer(model: Module, learning_rate: float, loss: Loss) -> Engine:
+    """
+    construct a trainer ``ignite`` engine with pre-attached progress bar and loss running average
+
+    :param model: a network to train
+    :param learning_rate: the learning rate for training
+    :param loss: a loss to minimise during training
+    :returns: an ``ignite`` trainer
+    """
+    trainer = create_supervised_trainer(
+        model,
+        Adam(model.parameters(), lr=learning_rate),
+        loss,
+        CURRENT_DEVICE,
+    )
+    RunningAverage(output_transform=lambda x: x).attach(
+        trainer, "running_loss"
+    )
+    ProgressBar().attach(
+        trainer,
+        output_transform=lambda x: x,
+        event_name=Events.EPOCH_COMPLETED,
+        closing_event_name=Events.COMPLETED,
+    )
+    return trainer
+
+
 def learning_pipeline(
     params: Dict[str, Union[int, float]],
     cardinality: int,
@@ -290,12 +338,7 @@ def learning_pipeline(
     :param loss: the criterion to optimize
     :param data_loaders: train, validation, and test data loaders
     """
-    trainer = create_supervised_trainer(
-        model,
-        Adam(model.parameters(), lr=params["learning_rate"]),
-        loss,
-        CURRENT_DEVICE,
-    )
+    trainer = get_trainer(model, params["learning_rate"], loss)
     evaluators = get_three_evaluators(model, loss)
 
     @trainer.on(Events.EPOCH_COMPLETED)
@@ -313,12 +356,6 @@ def learning_pipeline(
     add_early_stopping_and_checkpoint(
         evaluators.validation, trainer, f"semigroup{cardinality}", model
     )
-    ProgressBar().attach(
-        trainer,
-        output_transform=lambda x: x,
-        event_name=Events.EPOCH_COMPLETED,
-        closing_event_name=Events.COMPLETED,
-    )
     tb_logger = get_tensorboard_logger(trainer, evaluators)
     trainer.run(data_loaders[0], max_epochs=params["epochs"])
     tb_logger.close()
@@ -329,7 +366,7 @@ def get_loaders(
     batch_size: int,
     train_size: int,
     validation_size: int,
-    use_labels: bool = False,
+    dropout_rate: float,
 ) -> Tuple[DataLoader, DataLoader, DataLoader]:
     """
     get train and validation data loaders
@@ -338,8 +375,7 @@ def get_loaders(
     :param batch_size: batch size (common for train and validation)
     :param train_size: number of tables for training
     :param validation_size: number of tables for validation
-    :param use_labels: whether to set a target as labels from database (for
-    classifier) or to use X's as labels (for autoencoder)
+    :param dropout_rate: a dropout rate for validation and test set 'labels'
     :returns: a triple of train, validation, and test data loaders
     """
     (
@@ -349,15 +385,12 @@ def get_loaders(
         train_labels,
         validation_labels,
         test_labels,
-    ) = load_database_as_cubes(cardinality, train_size, validation_size)
-    if use_labels:
-        train_data = TensorDataset(train_tensor, train_labels)
-        val_data = TensorDataset(val_tensor, validation_labels)
-        test_data = TensorDataset(test_tensor, test_labels)
-    else:
-        train_data = TensorDataset(train_tensor, train_tensor)
-        val_data = TensorDataset(val_tensor, val_tensor)
-        test_data = TensorDataset(test_tensor, test_tensor)
+    ) = load_database_as_cubes(
+        cardinality, train_size, validation_size, dropout_rate
+    )
+    train_data = TensorDataset(train_tensor, train_labels)
+    val_data = TensorDataset(val_tensor, validation_labels)
+    test_data = TensorDataset(test_tensor, test_labels)
     return (
         DataLoader(train_data, batch_size=batch_size, shuffle=True),
         DataLoader(val_data, batch_size=batch_size, shuffle=True),
